@@ -30,7 +30,7 @@ struct gengetopt_args_info args_info;
 int sockfd;
 
 int
-writeln (char *str)
+writeln (const char *str)
 {
   printf ("%s\n", str);
 
@@ -103,7 +103,7 @@ select_mechanism (char **mechlist)
   else if (args_info.server_flag)
     {
       if (!args_info.quiet_given)
-	fprintf (stderr, _("Chose SASL mechanisms:\n"));
+	fprintf (stderr, _("Chose SASL mechanism:\n"));
       if (!readln (&in))
 	return 0;
       *mechlist = in;
@@ -299,7 +299,7 @@ main (int argc, char *argv[])
   res = gsasl_init (&ctx);
   if (res != GSASL_OK)
     {
-      fprintf (stderr, _("GSASL error (%d): %s\n"), res,
+      fprintf (stderr, _("Libgsasl error (%d): %s\n"), res,
 	       gsasl_strerror (res));
       return 1;
     }
@@ -344,15 +344,20 @@ main (int argc, char *argv[])
 
       if (res != GSASL_OK)
 	{
-	  fprintf (stderr, _("GSASL error (%d): %s\n"), res,
+	  fprintf (stderr, _("Libgsasl error (%d): %s\n"), res,
 		   gsasl_strerror (res));
 	  return 1;
 	}
 
       if (!args_info.quiet_given)
-	fprintf (stderr, _("This %s supports the following mechanisms:\n"),
-		 args_info.
-		 client_mechanisms_flag ? _("client") : _("server"));
+	{
+	  if (args_info.client_mechanisms_flag)
+	    fprintf (stderr,
+		     _("This client supports the following mechanisms:\n"));
+	  else
+	    fprintf (stderr,
+		     _("This server supports the following mechanisms:\n"));
+	}
 
       fprintf (stdout, "%s\n", mechs);
 
@@ -368,7 +373,6 @@ main (int argc, char *argv[])
       ssize_t b64output_len;
       const char *mech;
       Gsasl_session_ctx *xctx = NULL;
-      int res;
 
       if (!select_mechanism (&in))
 	return 1;
@@ -423,9 +427,14 @@ main (int argc, char *argv[])
 	no_client_first:
 	  if (!args_info.quiet_given &&
 	      !args_info.imap_flag && !args_info.smtp_flag)
-	    fprintf (stderr, _("Enter base64 authentication data from %s "
-			       "(press RET if none):\n"),
-		     args_info.client_flag ? _("server") : _("client"));
+	    {
+	      if (args_info.client_flag)
+		fprintf (stderr, _("Enter base64 authentication data "
+				   "from server (press RET if none):\n"));
+	      else
+		fprintf (stderr, _("Enter base64 authentication data "
+				   "from client (press RET if none):\n"));
+	    }
 
 	  if (!step_recv (&in))
 	    return 1;
